from typing import Any, Dict, Union
import pytest
from unittest.mock import MagicMock, patch
from pr_review_bot import analyze_pr, submit_review
from typer.testing import CliRunner
from pr_review_bot import app


runner = CliRunner()

@pytest.fixture
def mock_pr() -> MagicMock:
    pr = MagicMock()
    pr.body = "This is a test PR description."
    pr.number = 1
    return pr

@pytest.fixture
def mock_file() -> MagicMock:
    file = MagicMock()
    file.filename = "file1.txt"
    file.status = "added"
    file.patch = "This is a test patch."
    return file

def test_analyze_pr(mock_pr: MagicMock, mock_file: MagicMock) -> None:
    with patch("pr_review_bot.ghapi") as mock_ghapi:
        mock_ghapi.pulls.list_files.return_value = [mock_file]
        mock_ghapi.issues.list_comments.return_value = []

        with patch("pr_review_bot.openai") as mock_openai:
            response_mock = MagicMock()
            response_mock.choices = [
                    MagicMock(
                        message={
                            "role": "assistant",
                            "content": "This is a test review."
                        },
                        finish_reason="stop",
                        index=0
                    )
                ]
            response_mock.usage = {"prompt_tokens": 56, "completion_tokens": 31, "total_tokens": 87}
            mock_openai.ChatCompletion.create.return_value = response_mock

            review = analyze_pr(mock_pr)
            assert "This is a test review." in review["body"]


def test_submit_review(mock_pr: MagicMock) -> None:
    review = {"body": "This is a test review.", "event": "COMMENT"}

    with patch("pr_review_bot.ghapi") as mock_ghapi:
        submit_review(mock_pr, review)
        mock_ghapi.pulls.create_review.assert_called_once_with(mock_pr.number, body=review["body"], event=review["event"])


def test_review_all_open_pr(mock_pr: MagicMock) -> None:
    with patch("pr_review_bot.get_open_prs") as mock_get_open_prs:
        mock_get_open_prs.return_value = [mock_pr]
        
        with patch("pr_review_bot.analyze_pr") as mock_analyze_pr:
            mock_analyze_pr.return_value = {"body": "This is a test review.", "event": "COMMENT"}
            
            with patch("pr_review_bot.submit_review") as mock_submit_review:
                result = runner.invoke(app, ["review-all-open-pr"])
                assert f"Review submitted for PR #{mock_pr.number}" in result.output

def test_review_pr(mock_pr: MagicMock) -> None:
    with patch("pr_review_bot.ghapi") as mock_ghapi:
        mock_ghapi.pulls.get.return_value = mock_pr
        
        with patch("pr_review_bot.analyze_pr") as mock_analyze_pr:
            mock_analyze_pr.return_value = {"body": "This is a test review.", "event": "COMMENT"}
            
            with patch("pr_review_bot.submit_review") as mock_submit_review:
                result = runner.invoke(app, ["review-pr", str(mock_pr.number)])  # Convert input to string
                assert f"Review submitted for PR #{mock_pr.number}" in result.output





import openai
from ghapi.all import GhApi
from pydantic import BaseSettings
import typer
from typing import Optional, List, Dict, Any, Union

class Settings(BaseSettings):
    TOKEN: str
    OPEN_AI_KEY: str
    OWNER: str
    REPO_NAME: str
    PRICE_PER_TOKEN: float = 2.0000000000000002e-07
    MODEL_NAME: str = "gpt-3.5-turbo"

    class Config:
        env_file = '.env'
        env_prefix = "PR_REVIEW_BOT_"

settings = Settings()

openai.api_key = settings.OPEN_AI_KEY
ghapi = GhApi(token=settings.TOKEN, owner=settings.OWNER, repo=settings.REPO_NAME)

app = typer.Typer()

def get_open_prs() -> List[Any]:
    return ghapi.pulls.list(state='open')

def analyze_pr(pr: Any) -> Dict[str, Union[str, float]]:
    pr_description = pr.body

    # Read all PR files
    pr_files = ghapi.pulls.list_files(pr.number)
    pr_content = "\n".join(f"filename: {file.filename}: status: {file.status} patch: {file.patch}" for file in pr_files)

    # Read all PR comments
    comments = ghapi.issues.list_comments(pr.number)
    pr_comments = "\n".join(comment.body for comment in comments)

    text = f"pr_description\n {pr_description}, \npr_content\n = {pr_content}, \npr_comments\n = {pr_comments}"

    messages = [
        {"role": "system", "content": "You are github PR reviwer assistant."},
        {"role": "user", "content": f"Analyze this pull request text and provide a review:\n\n{text}"}
    ]

    response = openai.ChatCompletion.create(
        model=settings.MODEL_NAME,
        messages=messages,
    )

    review_cost = response['usage']['total_tokens'] * settings.PRICE_PER_TOKEN
    review = response.choices[0].message['content'].strip()
    review = f"Review \n\n{review}\n \n\nReview costs \n\n{review_cost} USD"

    event = "COMMENT"

    if "approve" in review.lower():
        event = "APPROVE"
    elif "request changes" in review.lower():
        event = "REQUEST_CHANGES"

    return {
        'body': review,
        'event': event
    }

def submit_review(pr: Any, review: Dict[str, Union[str, float]]) -> None:
    ghapi.pulls.create_review(pr.number, body=review['body'], event=review['event'])

@app.command()
def review_all_open_pr() -> None:
    open_prs = get_open_prs()
    for pr in open_prs:
        review = analyze_pr(pr)
        if review:
            submit_review(pr, review)
            typer.echo(f"Review submitted for PR #{pr.number}")

@app.command()
def review_pr(pr_number: Optional[int] = typer.Argument(None)) -> None:
    if pr_number is not None:
        pr = ghapi.pulls.get(pr_number)
        review = analyze_pr(pr)
        if review:
            submit_review(pr, review)
            typer.echo(f"Review submitted for PR #{pr.number}")
    else:
        typer.echo("Please provide a valid PR number.")

if __name__ == "__main__":
    app()
